/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.msm;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.msm.MetadataSet;
import org.greenstone.gatherer.msm.MetadataSetManager;
import org.greenstone.gatherer.util.Utility;
import org.greenstone.gatherer.gui.SimpleMenuBar;
import org.greenstone.gatherer.gui.ModalDialog;

/** A GUI component for allowing the user to export a metadata set, based on certain conditions, and to a certain file (not necessarily a .mds file). 
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
final public class ExportMDSPrompt
    extends ModalDialog 
    implements ActionListener, KeyListener {
    /** Is this an export prompt or an import one? */
    private boolean export;
    /** The file we wish to export the metadata set to. */
    private File file = null;
    /** The action the user has chosen from the dialog (either -1, if cancelled, or EXPORT). */
    private int action = -1;
    /** The button used to browse the local file system. */
    private JButton browse_button = null;
    /** Used to cancel the dialog. */
    private JButton cancel_button = null;
    /** Used to initiate the export, then dispose of the dialog. */
    private JButton export_button = null;
    /** The metadata sets available for export. */
    private JComboBox sets = null;
    /** The destination file name as a string. */
    private JTextField file_name = null;
    /** Ths button that, if selected, signifies you wish to export the metadata set with all values. */
    private JRadioButton all_values = null;
    /** The button that, if selected, signifies you wish to export the metadata set without any values (ie no mdv files). */
    private JRadioButton no_values = null;
    /** The button that, if selected, signifies you wish to export the metadata set with only those values that are subject nodes in the hierarchy. */
    private JRadioButton structure_only = null;
    /** A reference to the metadata set manager. */
    private MetadataSetManager manager;
    /** The default size for this dialog window. */
    final static public Dimension EXPORT_SIZE = new Dimension(500,245);
    /** The default size for this dialog window. */
    final static public Dimension IMPORT_SIZE = new Dimension(500,145);
    /** The default export action (there are several depending on how much information you wish to export). */
    final static public int EXPORT = 0;

    /** Constructor.
     * @param manager A reference to the <strong>MetadataSetManager</strong>.
     * @see org.greenstone.gatherer.Configuration
     * @see org.greenstone.gatherer.util.Utility
     */
    public ExportMDSPrompt(MetadataSetManager manager, boolean export) {
	super(Gatherer.g_man);
	this.export = export;

	// Creation
	setModal(true);
	if(export) {
	    setSize(EXPORT_SIZE);
	    setJMenuBar(new SimpleMenuBar("selectingmetadatasets"));
	    Dictionary.setText(this, "MSMPrompt.Export_Title");
	}
	else {
	    setSize(IMPORT_SIZE);
	    setJMenuBar(new SimpleMenuBar("selectingmetadatasets"));
	    Dictionary.setText(this, "MSMPrompt.Import_Title");
	}
	JPanel content_pane = (JPanel) getContentPane();
	JPanel control_pane = new JPanel();
	JLabel set_label = new JLabel();
	Dictionary.setText(set_label, "MSMPrompt.Export_Set");
	sets = new JComboBox(manager.getSets());
	Dictionary.setTooltip(sets, "MSMPrompt.Export_Set_Tooltip");
	JLabel condition_label = new JLabel();
	Dictionary.setText(condition_label, "MSMPrompt.Export_Conditions");
	JPanel condition_pane = new JPanel();
	ButtonGroup condition_group = new ButtonGroup();
	all_values = new JRadioButton();
	all_values.setOpaque(false);
	Dictionary.setText(all_values, "MSMPrompt.Export_All_Values");
	condition_group.add(all_values);
	no_values = new JRadioButton();
	no_values.setOpaque(false);
	Dictionary.setText(no_values, "MSMPrompt.Export_No_Values");
	condition_group.add(no_values);
	structure_only = new JRadioButton();
	structure_only.setOpaque(false);
	Dictionary.setText(structure_only, "MSMPrompt.Export_Subjects_Only");
	condition_group.add(structure_only);
	all_values.setSelected(true);
	JLabel file_label = new JLabel();
	Dictionary.setText(file_label, "MSMPrompt.Export_File");
	JPanel file_pane = new JPanel();
	file_name = new JTextField(Utility.METADATA_DIR);
	Dictionary.setTooltip(file_name, "MSMPrompt.Export_File_Tooltip");
	browse_button = new JButton();
	Dictionary.setBoth(browse_button, "General.Browse", "General.Browse_Tooltip");

	JPanel button_pane = new JPanel();
	if(export) {
	    export_button = new JButton();
	    export_button.setEnabled(false);
	    export_button.setMnemonic(KeyEvent.VK_E);
	    Dictionary.setBoth(export_button, "MSMPrompt.File_Export", "MSMPrompt.File_Export_Tooltip");
	}
	else {
	    export_button = new JButton();
	    export_button.setEnabled(true);
	    export_button.setMnemonic(KeyEvent.VK_I);
	    Dictionary.setBoth(export_button, "MSMPrompt.File_Import", "MSMPrompt.File_Import_Tooltip");
	}

	cancel_button = new JButton();
	cancel_button.setMnemonic(KeyEvent.VK_C);
	Dictionary.setBoth(cancel_button, "General.Cancel", "General.Pure_Cancel_Tooltip");

	// Listeners
	browse_button.addActionListener(this);
	cancel_button.addActionListener(this);
	export_button.addActionListener(this);
	file_name.addKeyListener(this);

	// Layout
	condition_pane.setLayout(new GridLayout(1,3));
	condition_pane.add(all_values);
	condition_pane.add(structure_only);
	condition_pane.add(no_values);

	file_pane.setLayout(new BorderLayout());
	file_pane.add(file_name, BorderLayout.CENTER);
	file_pane.add(browse_button, BorderLayout.EAST);

	if(export) {
	    control_pane.setLayout(new GridLayout(6, 1));
	    control_pane.add(set_label);
	    control_pane.add(sets);
	    control_pane.add(condition_label);
	    control_pane.add(condition_pane);
	    control_pane.add(file_label);
	    control_pane.add(file_pane);
	}
	else {
	    control_pane.setLayout(new GridLayout(2,1));
	    control_pane.add(condition_label);
	    control_pane.add(condition_pane);
	}
		  
	button_pane.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	button_pane.setLayout(new GridLayout(1,2));
	button_pane.add(export_button);
	button_pane.add(cancel_button);

	content_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	content_pane.setLayout(new BorderLayout());
	content_pane.add(control_pane, BorderLayout.CENTER);
	content_pane.add(button_pane, BorderLayout.SOUTH);
	// Display
	Dimension screen_size = Gatherer.config.screen_size;
	setLocation((screen_size.width - getSize().width) / 2, (screen_size.height - getSize().height) / 2);
    }

    /** Whenever one of the buttons in the dialog is actioned this method is called to trigger the appropriate effects.
     * @param event An <strong>ActionEvent</strong> containing information about the action performed.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.msm.MDSFileFilter
     * @see org.greenstone.gatherer.util.Utility
     */
    public void actionPerformed(ActionEvent event) {
	if(event.getSource() == export_button) {
	    action = EXPORT;
	    dispose();
	}
	else if(event.getSource() == cancel_button) {
	    action = -1;
	    dispose();
	}
	else {
	    JFileChooser chooser = new JFileChooser(new File(Utility.METADATA_DIR));
	    javax.swing.filechooser.FileFilter filter = new MDSFileFilter();
	    chooser.setApproveButtonText(Dictionary.get("General.OK"));
	    chooser.setFileFilter(filter);
	    int returnVal = chooser.showSaveDialog(Gatherer.g_man);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		file = chooser.getSelectedFile();
		if(file.getName().indexOf(".") == -1) {
		    file = new File(file.getName() + ".mds");
		}
		file_name.setText(file.getAbsolutePath());
		export_button.setEnabled(true);
	    }
	}
    }
    /** Show the prompt and get the user input. 
     * @return An <i>int</i> specifying what action the user has choosen.
     */
    public int display() {
	setVisible(true);
	return action;
    }
    /** Get the current value of condition.
     * @return The value as an <i>int</i>.
     * @see org.greenstone.gatherer.msm.MetadataSet
     */
    public int getSelectedCondition() {
	if(all_values.isSelected()) {
	    return MetadataSet.ALL_VALUES;
	}
	else if(no_values.isSelected()) {
	    return MetadataSet.NO_VALUES;
	}
	else {
	    return MetadataSet.SUBJECTS_ONLY;
	}
    }
    /** Get the current value of file.
     * @return The value as a <strong>File</strong>.
     */
    public File getSelectedFile() {
	return file;
    }
    /** Get the current value of set.
     * @return The value as a <strong>MetadataSet</strong>.
     */
    public MetadataSet getSelectedSet() {
	return (MetadataSet) sets.getSelectedItem();
    }
    /** Any implementation of KeyListener must include this method so that we can be informed when a key has been pressed. In this case we ignore it.
     * @param event A <strong>KeyEvent</strong> containing information about the key pressed.
     */
    public void keyPressed(KeyEvent event) {
    }
    /** Any implementation of KeyListener must include this method so that we can be informed once a key has been released. This is the earliest the VK code becomes stable and usable, so we will check if the file named in file_name can be written to and if so enable the export button.
     * @param event A <strong>KeyEvent</strong> containing information about the key typed.
     */
    public void keyReleased(KeyEvent event) {
	String pos_file = file_name.getText();
	if(pos_file.indexOf(".") != -1) {
	    file = new File(file_name.getText());
	    if(file.canWrite() || !file.exists()) {
		export_button.setEnabled(true);
	    }
	    else if(export) {
		export_button.setEnabled(false);
	    }
	}
    }
    /** Any implementation of KeyListener must include this method so that we can be informed when a key has been typed. In this case we ignore it.
     * @param event A <strong>KeyEvent</strong> containing information about the key typed.
     */
    public void keyTyped(KeyEvent event) {
    }
}
